// Determinarea numarului de muchii care trebuie adaugate unu graf
// CONEX pt a deveni eulerian.

// Solutie. Backtracking.
// Pastram in sirul de muchiii e[] toate muchiile care pot
// fi adaugate pana cand graful devine complet. Generam apoi cu back()
// submultimi posibile de muchii de adaugat. Aceste submultimi, cu numar din ce in ce mai
// mare de elemente (ma) se genereaza in sirul x[]. Pt fiecare submultime generata,
// in valid() se construiete in matricea temp[][] graful corespunzator,
// adaugand la graful dat muchiile noi si se verifica daca a devenit eulerian

#include <fstream.h>
#include <stdlib.h>
#include <iomanip.h>
#include <string.h>
//using namespace std;

const int DIM = 100;

struct muchie {
	int v1, v2;
} e[DIM];    // muchiile care trebuie adagate

int ma;      // nr muchii adaugate
int max_m;   // nr max de muchii pot fi adaugate pt un graf complet

int a[DIM][DIM];
int n;
int m;
int d[DIM];   // sirul gradelor
int x[DIM];   // retine muchiile care trebuie adaugate (etichetele lor)
int gasit;    // == 1 daca s-a gasit un graf eulerian
ofstream fout("graf.out");

void cit();
void euler();
void afis();
void back(int);
int valid(int);

int main()
{
	cit();
	euler();

	return 0;
}


void cit()
{
	int x, y;
	ifstream fin("graf.in");
	fin >> n >> m;
	for ( int i = 1; i <= m; i++ )
	{
		fin >> x >> y;
		d[x]++;
		d[y]++;
		a[x][y] = a[y][x] = 1;
	}
	fin.close();
}


void euler()
{
	int ok = 1;
	int i, j;
	for ( i = 1; i <= n && ok; i++ )
		if ( d[i] % 2  )
			ok = 0;
	if ( ok == 1 )
	{
		fout << " Graful de intrare este eulerian!";
		exit(0);
	}

	max_m = 0;   // nr max de muchii care pot fi adaugate, ca sa obtin un graf complet

	for ( i = 1; i < n; i++  )
		for ( j = i + 1; j <= n; j++ )
			if ( !a[i][j] )  // nu este muchie
			{
				e[++max_m].v1 = i;     // se poate adauga aceasta muchie
				e[max_m].v2   = j;
			}
	int gasit = 0;
	for ( ma = 1; ma <= max_m; ma++ ) // nr de muchii adaugate creste progresiv
		back(1);		              // pana graful devine eulerian
	                                  // Back() adauga ma muchii

	if ( !gasit ) fout << "Imposibil";
}

void grade(int a[][DIM])
{
	for ( int i = 1; i <= n; i++ )
	{
		d[i] = 0;
		for ( int j = 0; j <= n; j++ )
			d[i] += a[i][j];
	}
}

void back(int k)
{
	for ( int i = x[k-1] + 1; i <= max_m - ma + k; i++ )
	{
		x[k] = i;
		if ( valid(k) )
			if ( k == ma )
			{
				gasit = 1;
				afis();
				exit(0);
			}
			else back(k + 1);
	}
}

int valid(int k)
{
	if ( k < ma ) return 1;      // nu verific pana nu am pus destule muchii
	int temp[DIM][DIM];
	int j, b, v1, v2;
	memcpy(temp, a, sizeof(a));   // copiez matr de adiacenta in temp
	for ( j = 1; j <= k; j++ )    // adaug cele k muchii la graf
	{
		b = x[j];
		v1 = e[b].v1;
		v2 = e[j].v2;
		temp[v1][v2] = temp[v2][v1] = 1;
	}

	grade(temp);                      // si vad daca a devenit eulerian

	for ( j = 1; j <= n; j++ )
		if ( d[j] % 2 ) return 0;

	return 1;
}

void afis()
{
	int i, j;
	fout << "Muchille adaugate :\n";
	for ( i = 1; i <= ma; i++ )
		fout << e[i].v1 << " " << e[i].v2 << "\n";
}








