#include <stdio.h>
#define InFile "caini.in"
#define OutFile "caini.out"
#define MAXN  17
#define DoiLa ((1<<16)+1)
#define Inf 1000000000

typedef struct {int T[MAXN], mal;} Config;
/*Asociem problemei un graf neorientat astfel:
  multimea cainilor reprezinta multimea varfurilor grafului
  exista muchie intre nodul a si nodul b daca cainii a si b se dusmanesc.
  Vom reprezenta graful prin liste de adiacenta.
  G[a][0]=numarul de caini cu care se dusmaneste cainele a
  G[a][i]= al i -lea caine cu care se dusmaneste cainele a.
  Numim configuratie o multime de caini aflati pe un anumit mal.
  Mai exact o configuratie este o structura cu doua campuri:
  mal  care poate fi 0 sau 1
  T care este vectorul caracteristic al multimii cainilor situati pe malul mal.
  O astfel de reprezentare este costisitoare ca memorie, prin urmare,
  pentru a memora toate configuratiile corecte vom reprezenta o configuratie condensat,
  ca un nr.Putem considera vectorul caracteristic ca fiind reprezentarea binara a unui numar
  si retinem numarul respectiv in baza 10, la care adaugam malul drept cea mai semnificativa cifra din reprezentare.
  Vom genera prin backtracking toate configuratiile de caini astfel incat oricare
  doi caini din multime nu se dusmanesc si le vom retine in vectorul c.
  Pe parcursul generarii vom retine si max (numarul maxim de caini care nu se dusmanesc, deci
  care pot fi lasati singuri pe acelasi mal
*/

FILE *out;
int c[DoiLa];
/*int From[DoiLa];*/
int G[MAXN][MAXN];
int n;  /*numarul de caini */
int max=-1; /* numarul maxim de caini astfel incat oricare doi caini sa nu se dusmaneasca */
int nrc=0; /*numarul de configuratii din c */

int NrCaini=0;
Config S;
int Dusman[MAXN];
int ConfigToNr(Config);
void NrToConfig(int , Config * );
void Succesor(int);
void CitGraf(void);
int NrMinMutari(int);
void Afisare(int , int );

int ConfigToNr(Config S)
{int rez = 0, i;
 for (i=n; i > 0; i--) rez = rez*2+S.T[i];
 return rez*2+S.mal; }

void NrToConfig(int k, Config * S)
{int i;
 S->mal = k%2; k /= 2;
 for (i=1; i<=n; i++) {S->T[i] = k%2; k /= 2;}
}

void Succesor(int k)
/* cand apelam Succesor(k) in vectorul S sunt fixate pozitiile
   0, 1, ..., k-1 */
{int i;
 if (k > n)  /* am obtinut o multime de caini */
    {if (NrCaini > max) max = NrCaini;
	 S.mal = 0;      /* o plasez pe malul 0, apoi pe malul 1 si retin in c aceste configuratii */
	 c[nrc++] = ConfigToNr(S);
	 S.mal = 1;
	 c[nrc++] = ConfigToNr(S);
	}
    else /*generam in continuare */
    {S.T[k] = 0; Succesor(k+1);
     if (!Dusman[k]) /* cainele k nu are dusmani in configuratia curenta */
        {
	    for (i=1; i <= G[k][0]; i++) Dusman[G[k][i]]++;
	    S.T[k]=1; NrCaini++;
	    Succesor(k+1);
	    for (i=1; i <= G[k][0]; i++) Dusman[G[k][i]]--;
        NrCaini--;
	    }
     }
}

void CitGraf()
{
FILE *in = fopen(InFile,"rt");
int K, a, b, i;
fscanf(in,"%d %d",&n, &K);
for (i = 0; i < K; i++)
    {
	fscanf(in,"%d %d",&a, &b);
    G[a][0]++; G[a][G[a][0]] = b;
    G[b][0]++; G[b][G[b][0]] = a;
    }
fclose(in);
}

int NrMinMutari(int maxloc)
/* functia returneaza numarul minim de transporturi cu o barca de maxloc
locuri sau -1 daca nu este posibil */
{
int Dist[DoiLa], Q[DoiLa], p, u, i, k, j, start, cati;
Config S, S1;
/*Dist[i]=numarul de transporturi efectuate pentru a ajunge din configuratia 0 in configuratia i
  sau -1 daca nu este posibil sa ajungem de la configuratia 0 la configuratia i
  From[i]=configuratia din care am ajuns in configuratia i */

for (i=0; i<DoiLa; Dist[i++] = -1);
Dist[0] = 0; /*From[0] = -1;*/
/* parcurgem BFS incepand din configuratia 0, configuratia finala,
   pana ajungem la configuratia 1, cea initiala*/

Q[0] = 0; p = u = 0;

/* Q este coada, p indica primul element din coada, iar u ultimul */
while (p <= u)
      {
		k = Q[p++]; /*extrag prima configuratie */
		NrToConfig(k,&S);
		start = (S.mal) ? 0 : 1;    /*celalalt mal */
		for (i = start; i < nrc; i += 2) /* parcurg toate configratiile care corespund celuilalt mal */
            {
			int k1 = c[i];
			NrToConfig(k1,&S1);
			if (Dist[k1] == -1)
               {
                /* verific daca se poate ajunge de la configuratia k1 la configuratia k
                intr-un singur transport folosind o barca cu maxloc locuri */
				cati = 0; /*numar cati caini au fost transportati de pe malul configuratiei k1
                pe malul configuratiei k */
				for (j=1; j<=n; j++)
					if (S1.T[j])
                        {
						if (S.T[j]) { cati = Inf; break; }
					    }
					    else
                        if (!S.T[j]) cati++;
				if (cati <= maxloc) /*incap in barca cainii transportati? */
                    {
					Dist[k1] = Dist[k]+1; /*From[k1] = k;*/
					if (k1 == 1) return Dist[k1]; /*am transportat toti cainii */
					Q[++u] = k1;
				}
			}
		}
	}
return -1;
}

void Afisare(int minmut, int minloc)
{
/*int temp[DoiLa]; 
  int tnr = 0, crt;
int i, j;*/
FILE *out = fopen(OutFile,"wt");
fprintf(out,"%d %d\n", minloc, minmut);
/*Config S, S1;
for (i=1; i!=-1; i=From[i]) temp[tnr++] = i;
NrToConfig(temp[tnr-1], &S);
crt = 1;
for (i = tnr-2; i >= 0; i--)
    {
	fprintf(out,"%d: ", crt++);
	NrToConfig(temp[i], &S1);
	for (j=1; j<=n; j++)
		if (!S.T[j] && !S1.T[j]) fprintf(out,"%d ",j);
	fprintf(out,"\n");
	S = S1;
	}*/
fclose(out);
}

int main()
{
int t;
CitGraf();
/*generez toate configuratiile de caini care nu se ataca */
Succesor(1);
t = NrMinMutari(n-max);
if (t != -1) Afisare(t, n-max+1);
   else Afisare(NrMinMutari(n-max+1),n-max+2);
return 0;
}
